"""
AmericoDraws - A library for processing images for robotic drawing.

This library provides tools to convert images into drawing paths
suitable for robotic arms.
"""

__version__ = '0.1.2'
__author__ = 'Lucas Dantas'

# Import main functions for easy access
from .image_processor import process_image_for_robot